﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using HeuristicLab.Data;

namespace HeuristicLab.DataPreprocessing {
  public class DataGridLogic : IDataGridLogic {

    private IPreprocessingData preprocessingData;

    public DataGridLogic(IPreprocessingData preprocessingData) {
      this.preprocessingData = preprocessingData;
    }

    public int Rows {
      get {
        return preprocessingData.Rows;
      }
    }

    public int Columns {
      get {
        return preprocessingData.Columns;
      }
    }

    public IEnumerable<string> ColumnNames {
      get {
        return preprocessingData.VariableNames;
      }
    }

    public IEnumerable<string> RowNames {
      get {
        return Enumerable.Range(1, Rows).Select(n => n.ToString());
      }
    }

    public bool Validate(string value, out string errorMessage, int columnIndex) {
      if (columnIndex < 0 || columnIndex > preprocessingData.VariableNames.Count()) {
        throw new ArgumentOutOfRangeException("column index is out of range");
      }
      bool valid = false;
      string variableName = preprocessingData.GetVariableName(columnIndex);
      if (preprocessingData.IsType<double>(variableName)) {
        double val;
        valid = double.TryParse(value, out val);
        errorMessage = string.Empty;
        if (!valid) {
          errorMessage = "Invalid Value (Valid Value Format: \"" + FormatPatterns.GetDoubleFormatPattern() + "\")";
        }
      } else if (preprocessingData.IsType<string>(variableName)) {
        valid = value != null;
        errorMessage = string.Empty;
        if (!valid) {
          errorMessage = "Invalid Value (string must not be null)";
        }
      } else if (preprocessingData.IsType<DateTime>(variableName)) {
        DateTime date;
        valid = DateTime.TryParse(value, out date);
        errorMessage = string.Empty;
        if (!valid) {
          errorMessage = "Invalid Value (Valid Value Format: \"" + CultureInfo.CurrentCulture.DateTimeFormat + "\"";
        }
      } else {
        throw new ArgumentException("column with variableName: " + variableName + " contains a non supported type.");
      }

      return valid;
    }

    public string GetValue(int rowIndex, int columnIndex) {
      return preprocessingData.GetCellAsString(preprocessingData.GetVariableName(columnIndex), rowIndex);
    }

    public bool SetValue(string value, int rowIndex, int columnIndex) {
      string variableName = preprocessingData.GetVariableName(columnIndex);
      bool valid = false;
      if (preprocessingData.IsType<double>(variableName)) {
        double val;
        valid = double.TryParse(value, out val);
        if (valid) {
          preprocessingData.SetCell<double>(variableName, rowIndex, val);
        }
      } else if (preprocessingData.IsType<string>(variableName)) {
        valid = value != null;
        if (valid) {
          preprocessingData.SetCell<string>(variableName, rowIndex, value);
        }
      } else if (preprocessingData.IsType<DateTime>(variableName)) {
        DateTime date;
        valid = DateTime.TryParse(value, out date);
        if (valid) {
          preprocessingData.SetCell<DateTime>(variableName, rowIndex, date);
        }
      } else {
        throw new ArgumentException("column with variableName: " + variableName + " contains a non supported type.");
      }

      return valid;
    }
  }
}
